@@ -40,8 +40,6 @@
 #endif
 #include <limits.h>
 
-using namespace UCOMMON_NAMESPACE;
-
 #ifdef  _MSWINDOWS_
 int cpr_setenv(const char *sym, const char *val, int flag)
 {
@@ -178,8 +176,3 @@ extern "C" void __cxa_pure_virtual(void)
 
 #endif
 
-// vim: set ts=4 sw=4:
-// Local Variables:
-// c-basic-offset: 4
-// tab-width: 4
-// End:
