@@ -1,5 +1,6 @@
 // Copyright (C) 1999-2005 Open Source Telecom Corporation.
 // Copyright (C) 2006-2014 David Sugar, Tycho Softworks.
+// Copyright (C) 2014 Savoir-Faire Linux Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -329,10 +330,10 @@ protected:
      * @return 0 (errSuccess) on success, else error code.
      * @param ia address of multicast group to join.
      */
-    Error join(const ucommon::Socket::address &ia);
-    Error join(const IPV4Multicast &ia);
+    Error join(const ucommon::Socket::address &ia, int iface = 0);
+    inline Error join(const IPV4Multicast &ia) { return join(ucommon::Socket::address(getaddress(ia))); }
 #ifdef  CCXX_IPV6
-    Error join(const IPV6Multicast &ia);
+    inline Error join(const IPV6Multicast &ia, int iface = 0) { return join(ucommon::Socket::address(getaddress(ia)), iface); }
 #endif
 
     /**
@@ -341,10 +342,10 @@ protected:
      * @return 0 (errSuccess) on success, else error code.
      * @param ia address of multicast group to drop.
      */
-    Error drop(const ucommon::Socket::address &ia);
-    Error drop(const IPV4Multicast &ia);
+    Error drop(const ucommon::Socket::address &ia, int iface = 0);
+    Error drop(const IPV4Multicast &ia) { return drop(ucommon::Socket::address(getaddress(ia))); }
 #ifdef  CCXX_IPV6
-    Error drop(const IPV6Multicast &ia);
+    Error drop(const IPV6Multicast &ia, int iface = 0) { return drop(ucommon::Socket::address(getaddress(ia)), iface); }
 #endif
 
     /**
