@@ -30,7 +30,7 @@
 #include <ucommon/platform.h>
 #endif
 
-NAMESPACE_UCOMMON
+namespace ucommon {
 
 /**
  * A number manipulation class.  This is used to extract, convert,
@@ -68,7 +68,7 @@ public:
      * @return string buffer.
      */
     inline const char *c_str() const
-        {return buffer;};
+        {return buffer;}
 
     /**
      * Get value of string buffer as a long integer.
@@ -81,21 +81,21 @@ public:
      * @return long integer value of string buffer.
      */
     inline long operator()()
-        {return get();};
+        {return get();}
 
     /**
      * Cast string as long integer and get value of buffer.
      * @return long integer value of string buffer.
      */
     inline operator long()
-        {return get();};
+        {return get();}
 
     /**
      * Cast object as a string to retrieve buffer.
      * @return string buffer of value.
      */
     inline operator char*()
-        {return buffer;};
+        {return buffer;}
 
     /**
      * Assign a value to the number.  This rewrites the string buffer.
@@ -235,6 +235,6 @@ inline const T (max)(const T& v1, const T& v2)
     return ((v1 > v2) ? v1 : v2);
 }
 
-END_NAMESPACE
+} // namespace ucommon
 
 #endif
