@@ -717,6 +717,12 @@ public:
          */
         static bool isAny(const struct sockaddr *address);
 
+        /**
+         * Set the socket address to ADDR_ANY:
+         *   0.0.0.0 or ::0
+         */
+        static void setAny(struct sockaddr *sa);
+
         /**
          * Get a ADDR_ANY socket address of the given family.
          */
@@ -729,6 +735,13 @@ public:
          */
         static bool isLoopback(const struct sockaddr *address);
 
+        /**
+         * Set the socket address to ADDR_LOOPBACK:
+         *   127.0.0.1 or ::1
+         * depending on the family of the pointed address.
+         */
+        static void setLoopback(struct sockaddr *sa);
+
         /**
          * Get a ADDR_LOOPBACK socket address of the given family.
          */
@@ -1952,19 +1965,6 @@ public:
         {ptr = _nextaddrinfo(ptr);}
 };
 
-#if (defined(OLD_STDCPP) || defined(NEW_STDCPP)) && !defined(_MSC_VER)
-inline std::ostream& operator<<(std::ostream& os, Socket::address& addr) {
-#ifdef  AF_INET6
-    char buf[INET6_ADDRSTRLEN];
-#else
-    char buf[INET_ADDRSTRLEN];
-#endif
-    size_t s = addr.print(buf, sizeof(buf), false, true);
-    os << std::string(buf, s);
-    return os;
-}
-#endif
-
 /**
  * A convenience function to convert a socket address list into an addrinfo.
  * @param address list object.
